<?php
/**
 * @package	com_abook
 * @copyright Federica Ugolotti. All rights reserved.
 * @license	http://www.gnu.org/licenses/gpl.html GNU/GPL
**/

use Alledia\OSMap\Factory;
use Alledia\OSMap\Helper\General;
use Alledia\OSMap\Plugin\Base;
use Alledia\OSMap\Plugin\ContentInterface;
use Alledia\OSMap\Sitemap\Collector;
use Alledia\OSMap\Sitemap\Item;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;

defined('_JEXEC') or die();

JLoader::register('AbookHelperRoute', JPATH_SITE . '/components/com_abook/helpers/route.php');
//JLoader::register('ContentHelperQuery', JPATH_SITE . '/components/com_content/helpers/query.php');

/**
 * Handles standard Abook's Content books/categories
 *
 * This plugin is able to expand the categories keeping the right order of the
 * books according to the menu settings and the user session data (user state).
 *
 * This is a very complex plugin, if you are trying to build your own plugin
 * for other component, I suggest you to take a look to another plugis as
 * they are usually most simple. ;)
 */
class PlgOSMapAbook extends Base implements ContentInterface
{
    /**
     * @var self
     */
    protected static $instance = null;

    /**
     * @var bool
     */
    protected static $prepareContent = null;

    /**
     * Returns the unique instance of the plugin
     *
     * @return self
     */
    public static function getInstance()
    {
        if (empty(static::$instance)) {
            $dispatcher       = Factory::getDispatcher();
            static::$instance = new self($dispatcher);
        }

        return static::$instance;
    }

    /**
     * Returns the element of the component which this plugin supports.
     *
     * @return string
     */
    public function getComponentElement()
    {
        return 'com_abook';
    }

    /**
     * This function is called before a menu item is used. We use it to set the
     * proper uniqueid for the item
     *
     * @param Item     $node   Menu item to be "prepared"
     * @param Registry $params The extension params
     *
     * @return bool
     * @throws Exception
     */
    public static function prepareMenuItem($node, $params)
    {
        static::checkMemory();

        $db        = Factory::getDbo();
        $container = Factory::getPimpleContainer();

        $linkQuery = parse_url($node->link);

        if (!isset($linkQuery['query'])) {
            return false;
        }

        parse_str(html_entity_decode($linkQuery['query']), $linkVars);

        $view = ArrayHelper::getValue($linkVars, 'view', '');
        $id   = ArrayHelper::getValue($linkVars, 'id', 0);

        switch ($view) {
            case 'archive':
                $node->adapterName = 'AbookCategory';
                $node->uid         = 'abook.archive.' . $node->id;
                $node->expandible  = true;

                break;
            case 'categories':
            case 'category':
                $node->adapterName = 'AbookCategory';
                $node->uid         = 'abook.category.' . $id;
                $node->expandible  = true;

                break;

            case 'book':
                $node->adapterName = 'AbookBook';
                $node->expandible  = false;

                $paramAddPageBreaks = $params->get('add_pagebreaks', 1);
                $paramAddImages     = $params->get('add_images', 1);

                $query = $db->getQuery(true)
                    ->select([
                        $db->quoteName('dateinsert'),
                        $db->quoteName('metadata'),
                        $db->quoteName('params')
                    ])
                    ->from($db->quoteName('#__abbook'))
                    ->where($db->quoteName('id') . '=' . (int)$id);

                if ($paramAddPageBreaks || $paramAddImages) {
                    $query->select([
                        $db->quoteName('description'),
                        $db->quoteName('image') . ' AS images'
                    ]);
                }

                $db->setQuery($query);

                if (($item = $db->loadObject()) != null) {
                    // Set the node UID
                    $node->uid = 'abook.book.' . $id;

                    // Set dates
                    $node->modified  = $item->dateinsert;
                    $node->created   = $item->dateinsert;
                    $node->publishUp = $item->dateinsert;

                    $item->params = $item->params;

                    $text = '';
                    if (isset($item->description)) {
                        $text = $item->description;
                    }

                    static::prepareContent($text, $params);

                    if ($paramAddImages) {
                        $maxImages = $params->get('max_images', 1000);

                        $node->images = [];

                        // Images from text
                        $node->images = array_merge(
                            $node->images,
                            $container->imagesHelper->getImagesFromText($text, $maxImages)
                        );

                        // Images from params
                        if (!empty($item->images)) {
			    $item->images = '{"image_intro":"","float_intro":"","image_intro_alt":"","image_intro_caption":"","image_fulltext":"'.$item->images.'","float_fulltext":"","image_fulltext_alt":"","image_fulltext_caption":""}';
                            $node->images = array_merge(
                                $node->images,
                                $container->imagesHelper->getImagesFromParams($item)
                            );
                        }
                    }

                    if ($paramAddPageBreaks) {
                        $node->subnodes   = General::getPagebreaks($text, $node->link, $node->uid);
                        $node->expandible = (count($node->subnodes) > 0); // This book has children
                    }
                } else {
                    return false;
                }

                break;
        }

        return true;
    }

    /**
     * Expands a com_content menu item
     *
     * @param Collector $collector
     * @param Item      $parent
     * @param Registry  $params
     *
     * @return void
     * @throws Exception
     */
    public static function getTree($collector, $parent, $params)
    {
        $db = Factory::getDbo();

        $linkQuery = parse_url($parent->link);

        if (!isset($linkQuery['query'])) {
            return;
        }

        parse_str(html_entity_decode($linkQuery['query']), $linkVars);
        $view = ArrayHelper::getValue($linkVars, 'view', '');
        $id   = intval(ArrayHelper::getValue($linkVars, 'id', ''));

        /*
         * Parameters Initialisation
         */
        $paramExpandCategories = $params->get('expand_categories', 1) > 0;
	$paramIncludeArchived  = $params->get('include_archived', 2);

        $paramAddPageBreaks = $params->get('add_pagebreaks', 1);

        $paramCatPriority   = $params->get('cat_priority', $parent->priority);
        $paramCatChangefreq = $params->get('cat_changefreq', $parent->changefreq);

        if ($paramCatPriority == '-1') {
            $paramCatPriority = $parent->priority;
        }
        if ($paramCatChangefreq == '-1') {
            $paramCatChangefreq = $parent->changefreq;
        }
        $params->set('cat_priority', $paramCatPriority);
        $params->set('cat_changefreq', $paramCatChangefreq);

        $paramArtPriority   = $params->get('art_priority', $parent->priority);
        $paramArtChangefreq = $params->get('art_changefreq', $parent->changefreq);

        if ($paramArtPriority == '-1') {
            $paramArtPriority = $parent->priority;
        }

        if ($paramArtChangefreq == '-1') {
            $paramArtChangefreq = $parent->changefreq;
        }

        $params->set('art_priority', $paramArtPriority);
        $params->set('art_changefreq', $paramArtChangefreq);

        switch ($view) {
            case 'category':
                if (empty($id)) {
                    $id = intval($params->get('id', 0));
                }

                if ($paramExpandCategories && $id) {
                    static::expandCategory($collector, $parent, $id, $params, $parent->id);
                }

                break;

            case 'categories':
                if ($paramExpandCategories) {
                    if (empty($id)) {
                        $id = 1;
                    }

                    static::expandCategory($collector, $parent, $id, $params, $parent->id);
                }

                break;

            case 'archive':
                if ($paramIncludeArchived) {
                    static::includeCategoryContent($collector, $parent, 'archived', $params);
                }

                break;

            case 'book':
                // if it's an book menu item, we have to check if we have to expand the
                // book's page breaks
                if ($paramAddPageBreaks) {
                    $query = $db->getQuery(true)
                        ->select([
                            $db->quoteName('description'),
                            $db->quoteName('alias'),
                            $db->quoteName('catid'),
                            $db->quoteName('params'),
                            $db->quoteName('metadata'),
                            $db->quoteName('dateinsert'),
                        ])
                        ->from($db->quoteName('#__abbook'))
                        ->where($db->quoteName('id') . '=' . $id);
                    $db->setQuery($query);

                    $item = $db->loadObject();

                    $item->uid = 'abook.book.' . $id;

                    $parent->slug = $item->alias ? ($id . ':' . $item->alias) : $id;
                    $parent->link = AbookHelperRoute::getBookRoute($parent->slug, $item->catid);

                    $parent->subnodes = General::getPagebreaks(
                        $item->description,
                        $parent->link,
                        $item->uid
                    );
                    static::printSubNodes($collector, $parent, $params, $parent->subnodes, $item);
                }
        }
    }

    /**
     * Get all content items within a content category.
     * Returns an array of all contained content items.
     *
     * @param Collector $collector
     * @param Item      $parent the menu item
     * @param int       $catid  the id of the category to be expanded
     * @param Registry  $params parameters for this plugin on Xmap
     * @param int       $itemid the itemid to use for this category's children
     * @param int       $curlevel
     *
     * @return void
     * @throws Exception
     */
    protected static function expandCategory(
        $collector,
        $parent,
        $catid,
        $params,
        $itemid,
        $curlevel = 0
    ) {
        static::checkMemory();

        $db = Factory::getDbo();

        $where = [
            'a.parent_id = ' . $catid,
            'a.published = 1',
            'a.extension=' . $db->quote('com_abook')
        ];

        if (!$params->get('show_unauth', 0)) {
            $where[] = 'a.access IN (' . General::getAuthorisedViewLevels() . ') ';
        }

        $query = $db->getQuery(true)
            ->select([
                'a.id',
                'a.title',
                'a.alias',
                'a.access',
                'a.path AS route',
                'a.created_time AS created',
                'a.modified_time AS modified',
                'a.params',
                'a.metadata',
                'a.metakey'
            ])
            ->from('#__abcategories AS a')
            ->where($where)
            ->order('a.lft');

        $items = $db->setQuery($query)->loadObjectList();

        $curlevel++;

        $maxLevel = $parent->params->get('max_category_level', 100);

        if ($curlevel <= $maxLevel) {
            if (count($items) > 0) {
                $collector->changeLevel(1);

                foreach ($items as $item) {
                    $node = (object)[
                        'id'                       => $item->id,
                        'uid'                      => 'abook.category.' . $item->id,
                        'browserNav'               => $parent->browserNav,
                        'priority'                 => $params->get('cat_priority'),
                        'changefreq'               => $params->get('cat_changefreq'),
                        'name'                     => $item->title,
                        'expandible'               => true,
                        'secure'                   => $parent->secure,
                        'newsItem'                 => 1,
                        'adapterName'              => 'AbookCategory',
                        'pluginParams'             => &$params,
                        'parentIsVisibleForRobots' => $parent->visibleForRobots,
                        'created'                  => $item->created,
                        'modified'                 => $item->modified,
                        'publishUp'                => $item->created
                    ];

                    // Keywords
                    $paramKeywords = $params->get('keywords', 'metakey');
                    $keywords      = null;
                    if ($paramKeywords !== 'none') {
                        $keywords = $item->metakey;
                    }
                    $node->keywords = $keywords;

                    $node->slug   = $item->route ? ($item->id . ':' . $item->route) : $item->id;
                    $node->link   = AbookHelperRoute::getCategoryRoute($node->slug);
                    $node->itemid = $itemid;

                    // Correct for an issue in Abook core with occasional empty variables
                    $linkUri = new Uri($node->link);
                    $linkUri->setQuery(array_filter((array)$linkUri->getQuery(true)));
                    $node->link = $linkUri->toString();

                    if ($collector->printNode($node)) {
                        static::expandCategory($collector, $parent, $item->id, $params, $node->itemid, $curlevel);
                    }
                }

                $collector->changeLevel(-1);
            }
        }

        // Include Category's content
        static::includeCategoryContent($collector, $parent, $catid, $params);
    }

    /**
     * Get all content items within a content category.
     * Returns an array of all contained content items.
     *
     * @param Collector  $collector
     * @param Item       $parent
     * @param int|string $catid
     * @param Registry   $params
     *
     * @return void
     * @throws Exception
     *
     */
    protected static function includeCategoryContent($collector, $parent, $catid, $params)
    {
        static::checkMemory();

        $db        = Factory::getDbo();
        $container = Factory::getPimpleContainer();

        $nullDate = $db->quote($db->getNullDate());
        $nowDate  = $db->quote(Factory::getDate()->toSql());

        $selectFields = [
            'a.id',
            'a.title',
            'a.alias',
            'a.catid',
            'a.dateinsert',
            'a.params',
            'a.metadata',
            'a.language',
            'a.metakey',
            'a.image',
            'c.title AS categMetakey'
        ];
        if ($params->get('add_images', 1) || $params->get('add_pagebreaks', 1)) {
            $selectFields[] = 'a.description';
        }

        if ($params->get('include_archived', 2)) {
            $where = ['(a.state = 1 or a.state = 2)'];
        } else {
            $where = ['a.state = 1'];
        }

        if ($catid == 'archived') {
            $where = ['a.state=2'];
        } elseif (is_numeric($catid)) {
            $where[] = 'a.catid=' . (int)$catid;
        }

        if (!$params->get('show_unauth', 0)) {
            $where[] = 'a.access IN (' . General::getAuthorisedViewLevels() . ') ';
        }

        //@todo: Do we need join for frontpage?
        $query = $db->getQuery(true)
            ->select($selectFields)
            ->from('#__abbook AS a')
            ->join('LEFT', '#__abcategories AS c ON (a.catid = c.id)')
            ->where($where);

        // Ordering
        $orderOptions    = [
            'a.dateinsert',
            'a.hits',
            'a.title',
            'a.ordering'
        ];
        $orderDirOptions = [
            'ASC',
            'DESC'
        ];

        $order    = ArrayHelper::getValue($orderOptions, $params->get('book_order', 0), 0);
        $orderDir = ArrayHelper::getValue($orderDirOptions, $params->get('book_orderdir', 0), 0);

        $orderBy = ' ' . $order . ' ' . $orderDir;
        $query->order($orderBy);

        $maxArt = (int)$params->get('max_art');
        $db->setQuery($query, 0, $maxArt);

        $items = $db->loadObjectList();

        if (count($items) > 0) {
            $collector->changeLevel(1);

            $paramExpandCategories = $params->get('expand_categories', 1);
            $paramIncludeArchived  = $params->get('include_archived', 2);

            foreach ($items as $item) {
                $node = (object)[
                    'id'                       => $item->id,
                    'uid'                      => 'abook.book.' . $item->id,
                    'browserNav'               => $parent->browserNav,
                    'priority'                 => $params->get('art_priority'),
                    'changefreq'               => $params->get('art_changefreq'),
                    'name'                     => $item->title,
                    'created'                  => $item->dateinsert,
                    'expandible'               => false,
                    'secure'                   => $parent->secure,
                    'newsItem'                 => 1,
                    'language'                 => $item->language,
                    'adapterName'              => 'AbookBook',
                    'parentIsVisibleForRobots' => $parent->visibleForRobots
                ];

                $keywords = [];

                $paramKeywords = $params->get('keywords', 'metakey');
                if ($paramKeywords !== 'none') {
                    if (in_array($paramKeywords, ['metakey', 'both'])) {
                        $keywords[] = $item->metakey;
                    }

                    if (in_array($paramKeywords, ['category', 'both'])) {
                        $keywords[] = $item->categMetakey;
                    }
                }
                $node->keywords = join(',', $keywords);

                $node->slug    = $item->alias ? ($item->id . ':' . $item->alias) : $item->id;
                $node->catslug = $item->catid;
                $node->link    = AbookHelperRoute::getBookRoute($node->slug, $node->catslug);

                // Set the visibility for XML or HTML sitempas
                if ($catid == 'archived') {
                    // Check if the item is visible in the XML or HTML sitemaps
                    $node->visibleForXML  = in_array($paramIncludeArchived, [1, 2]);
                    $node->visibleForHTML = in_array($paramIncludeArchived, [1, 3]);
                } elseif (is_numeric($catid)) {
                    // Check if the item is visible in the XML or HTML sitemaps
                    $node->visibleForXML  = in_array($paramExpandCategories, [1, 2]);
                    $node->visibleForHTML = in_array($paramExpandCategories, [1, 3]);
                }

                // Add images to the book
                $text = '';
                if (isset($item->description)) {
                    $text = $item->description;
                }

                if ($params->get('add_images', 1)) {
                    $maxImages = $params->get('max_images', 1000);

                    $node->images = [];

                    // Images from text
                    $node->images = array_merge(
                        $node->images,
                        $container->imagesHelper->getImagesFromText($text, $maxImages)
                    );

                    // Images from params
                    if (!empty($item->images)) {
                        $node->images = array_merge(
                            $node->images,
                            $container->imagesHelper->getImagesFromParams($item)
                        );
                    }
                }

                if ($collector->printNode($node) && $node->expandible) {
                    static::printSubNodes($collector, $parent, $params, $node->subnodes, $node);
                }
            }

            $collector->changeLevel(-1);
        }
    }

    /**
     * @param Collector $collector
     * @param Item      $parent
     * @param Registry  $params
     * @param array     $subnodes
     * @param object    $item
     *
     * @return void
     * @throws Exception
     */
    protected static function printSubNodes($collector, $parent, $params, $subnodes, $item)
    {
        static::checkMemory();

        $collector->changeLevel(1);

        $i = 0;
        foreach ($subnodes as $subnode) {
            $i++;

            $subnode->browserNav = $parent->browserNav;
            $subnode->priority   = $params->get('art_priority');
            $subnode->changefreq = $params->get('art_changefreq');
            $subnode->secure     = $parent->secure;
            $subnode->created    = $item->dateinsert;
            $subnode->modified   = $item->dateinsert;
            $subnode->publishUp  = $item->dateinsert;

            $collector->printNode($subnode);

            $subnode = null;
            unset($subnode);
        }

        $collector->changeLevel(-1);
    }

    /**
     * @param string   $text
     * @param Registry $params
     */
    protected static function prepareContent(&$text, Registry $params)
    {
        if (static::$prepareContent === null) {
            $isPro   = Factory::getExtension('osmap', 'component')->isPro();
            $isHtml  = Factory::getDocument()->getType() == 'html';
            $prepare = $params->get('prepare_content', true);

            static::$prepareContent = $isPro && $prepare && $isHtml;
        }

        if (static::$prepareContent) {
            $text = HTMLHelper::_('content.prepare', $text);
        }
    }
}
